classdef Face <hgsetget
    % input/output class for Agent
    %
    % Syntax
    %
    %       Agent.Face.who()
    %       Agent.Face.who(do,data)
    %       Agent.Face.plot(figH,Position,type,overwrite)
    %       Agent.Face.getMM(normalize)
    %       Agent.Face.getKN()
    %
    % Description
    %
    %     The Face of an agent handles creating,setting, saving and displaying
    %     agent parameters.
    %
    % Input
    %
    %   Agent.Face.who(do, data)
    %
    %   do          'set' to set agent parameters with either DATA as a struct
    %               file or with data as a path to a struct file.
    %               'save' to save an agent parameters to a struct file with
    %               data being a path.
    %
    %   Agent.Face.plot(figH,Position,type,overwrite)
    %
    %   figH        a handle to a figure
    %
    %   Position    the panel to plot to using subplot syntax [rows,cols,panel]
    %
    %   type        'MM' to plot the current mental model
    %               'KN' to plot the current knowledge regions
    %               'MMvKN' to plot current mental model with areas without
    %                   knowledge blackened out
    %               'ExperiencesTrace' to plot past experiences with experience
    %               age indicated by blacker and smaller circles
    %   overwrite   if true then panel will be set to hold off before plotting,
    %               else if false, panel will be set to hold on before plotting
    %
    %   Agent.Face.getMM(normalize)
    %
    %   normalize   'normalize' if values along action should be normalized to be 1 at
    %               maximum value. 'absolute' if plotting absolute values
    %
    % Author: Jon Norberg (jon.norberg[at]ecology.su.se) Date: 1. November,
    % 2012
    
    properties
        MM
        MMKN
        KN
        bA
        a
        I1
        S
    end
    properties (SetAccess='private')
        agent;
    end
    methods
        function obj=Face(agent,file)
            %Face(AGENT,FILE) creates classes and sets all parameters in
            %AGENT according to FILE which can be struct or path
            %
            %Calls setAgent(FILE)
            obj.agent=agent;
            obj.setAgent(file);
%            [obj.a,obj.I1]=meshgrid(linspace(0,1,obj.agent.policy.numTests));
        end
        
        function setAgent(obj,data)
            %setAgent(DATA) creates classes and sets all parameters in
            %according to FILE which can be struct or path
            if isstruct(data)
                S=data;
            elseif exist(data)
                z=load(data,'S');
                S=z.S;
            else
                disp('No struct or path given')
            end
            obj.agent.numInputs=S.Dimensions;
            obj.agent.numActions=S.numActions;
            obj.agent.numStates=S.Dimensions-S.numActions;
            
            %Create Neural Network class
            obj.agent.nn=RBFNeuralNetwork(S.numRbfs,repmat([0 1],S.Dimensions,1),S.learningRate,S.NeuronWidth);
            obj.agent.kn=RBFNeuralNetwork(S.numRbfs,repmat([0 1],S.Dimensions,1),S.learningRate,S.NeuronWidth);
            obj.agent.kn.setadjBias(false);
            obj.agent.kn.setSpread(7);
            
            %Create RL class
            obj.agent.useRL = true;
            obj.agent.lm=ReinforcementLearningModel ();
            obj.agent.lm.init (obj.agent);
            obj.agent.lm.setDiscountRate(S.discountRate);
            obj.agent.lm.setStepSize(S.stepSize);%OK, do we need this, how entangled in learning rate?
            obj.agent.lm.setUseEt(S.useTrace);
            
            %Create Trace
            obj.agent.et = EligibilityTrace();
            obj.agent.et.setLambda(S.lambda);
            
            %Create Policy
            obj.agent.policy = PolicyModel (obj.agent);
            obj.agent.policy.temperature = S.temperature;
            %obj.agent.policy.policyStrategy = S.policyStrategy;
            %obj.agent.policy.numTests = S.ActionResolution;
            obj.agent.policy.possibleActions = linspace (0, 1, obj.agent.policy.numTests);
            [obj.a,obj.I1]=meshgrid(linspace(0,1,obj.agent.policy.numTests));
            
            %Create Experiences
            obj.agent.experiences = Experiences();
        end
        
        function saveAgent(obj,path)
            %saveAgent(PATH) Saves agent parameters to PATH that can be
            %used to set up a new agent
            S=obj.getS();
            save(path,'S');
        end
        
        function setParams(obj,learningRate,discountRate, stepSize,lambda,...
                temperature, spread,numRbfs, RL, ET)
            %sets numrbfs,spread,learningRate,discountRate, stepSize,
            %lambda, temperature in the agent
            S=obj.getS();
            S.numRbfs=numRbfs;
            S.learningRate=learningRate;
            S.NeuronWidth=spread;
            S.discountRate=discountRate;
            S.stepSize=stepSize;
            S.lambda=lambda;
            S.temperature=temperature;
            S.useRL = RL;
            S.useTrace = ET;
            
            obj.setAgent(S);
        end
        
        function obj=who(obj)
            %who() Displays current parameter values in teh agent
            S.AGENT='-----------------------';
            S.ActionResolution=obj.agent.policy.numTests;
            S.Dimensions=obj.agent.numInputs;
            S.numActions=obj.agent.numActions;
            S.NEURAL_NETWORK='-----------------------';
            S.numRbfs=obj.agent.nn.numRbfs;
            S.learningRate=obj.agent.nn.lr;
            S.NeuronWidth=obj.agent.nn.spread;
            S.REINFORCEMENT_LEARNING='-----------------------';
            S.discountRate=obj.agent.lm.dr;
            S.stepSize=obj.agent.lm.ss;
            S.useRL=obj.agent.useRL;
            S.ELIGIBILITYTRACE='-----------------------';
            S.useTrace=obj.agent.lm.getUseEt();
            S.lambda=obj.agent.et.lambda;
            S.POLICY='-----------------------';
            S.temperature=obj.agent.policy.temperature;
            %S.policyStrategy=obj.agent.policy.policyStrategy;
            S
        end
        
        function S=getS(obj)
            %S=getS() Retreives the parameters of the agent as a struct file 
            S.AGENT='-----------------------';
            S.ActionResolution=obj.agent.policy.numTests;
            S.Dimensions=obj.agent.numInputs;
            S.numActions=obj.agent.numActions;
            S.NEURAL_NETWORK='-----------------------';
            S.numRbfs=obj.agent.nn.numRbfs;
            S.learningRate=obj.agent.nn.lr;
            S.NeuronWidth=obj.agent.nn.spread;
            S.REINFORCEMENT_LEARNING='-----------------------';
            S.discountRate=obj.agent.lm.dr;
            S.stepSize=obj.agent.lm.ss;
            S.useRL=obj.agent.useRL;
            S.ELIGIBILITYTRACE='-----------------------';
            S.useTrace=obj.agent.lm.getUseEt();
            S.lambda=obj.agent.et.lambda;
            S.POLICY='-----------------------';
            S.temperature=obj.agent.policy.temperature;
            %S.policyStrategy=obj.agent.policy.policyStrategy;
        end
        
        function output = Plot(obj, figH,Position,type,overwrite)
            %Build in plotting 
            figure(figH)
            %subplot(Position(1),Position(2),Position(3))
            if strcmp(overwrite,'on')
                hold on
            else
                hold off
            end
            if strcmp(type,'MMvKN')
                obj.getMM('normalize');
                obj.getKN();
                image([0 1],[0 1],(MakeRGB(obj.MM,obj.KN,64,02)))
                set(gca,'YDir','normal')
                axis([0 1 0 1])
                %pcolor(a,I1,q./repmat(max(q,[],1),obj.agent.policy.numTests,1));shading flat%./repmat(max(q,[],1),obj.numTests,1))'
                hold on
                [v,id]=max(obj.MM,[],1);
                plot(obj.I1(:,1),obj.a(1,id),'w-')
                
                L=length(obj.agent.experiences.states);
                Lmin=max(1,L-20);
                for i=Lmin:L
                    dL=max(0,L-20);
                    c=(i-dL)/(L-dL);
                    plot(obj.agent.experiences.states(i),obj.agent.experiences.actions(i),'o','MarkerEdgeColor','k','MarkerFaceColor',[c c c],'MarkerSize',c*10+5)
                    if i>1
                        %plot([obj.agent.experiences.states(i-1),obj.agent.experiences.states(i)],[obj.agent.experiences.actions(i-1),obj.agent.experiences.actions(i)],':w')
                    end
                end
                hold off
                colorbar
            elseif strcmp(type,'MM')
                obj.getMM('normalize');
                pcolor(obj.a,obj.I1,obj.MM./repmat(max(obj.MM,[],1),obj.agent.policy.numTests,1));shading flat
            elseif strcmp(type,'KN')
                obj.getKN();
                pcolor(obj.a,obj.I1,obj.KN);shading flat%./repmat(max(q,[],1),obj.numTests,1))'
            elseif strcmp(type,'ExperiencesTrace')
                L=length(obj.agent.experiences.states);
                Lmin=max(1,L-20);
                for i=Lmin:L
                    dL=max(0,L-20);
                    c=(i-dL)/(L-dL);
                    plot(obj.agent.experiences.states(i),obj.agent.experiences.actions(i),'o','MarkerEdgeColor','k','MarkerFaceColor',[c c c],'MarkerSize',c*10+5)
                    if i>1
                        plot([obj.agent.experiences.states(i-1),obj.agent.experiences.states(i)],[obj.agent.experiences.actions(i-1),obj.agent.experiences.actions(i)],':w')
                    end
                end
            elseif strcmp(type,'StateAction')
                plot(obj.agent.experiences.trainingNo,obj.agent.experiences.actions,'r',obj.agent.experiences.trainingNo,obj.agent.experiences.states,'b')
                legend('Actions','STates')
                ylim([0 1])
            end
        end
        
        
        function output = getMM(obj,normalize)
            %Creates a matrix of the Neural Network over state and action space
            [a,I1]=meshgrid(linspace(0,1,obj.agent.policy.numTests));
            for i=1:obj.agent.policy.numTests
                for j=1:obj.agent.policy.numTests
                    q(j,i)=obj.agent.nn.sim([a(i,j);I1(i,j)]);
                    k(j,i)=obj.agent.kn.sim([a(i,j);I1(i,j)]);
                end
            end
            q=q+max(abs(min(min(q))));
            if strcmp(normalize,'normalize')
                q=q./repmat(max(0.00000001,max(q,[],1)),obj.agent.policy.numTests,1);
            end
            [val id]=max(q,[],2);
            obj.bA=a(:,id);
            obj.MM=q;
            obj.KN=k;
            output=q;
        end
        
        function output = getKN(obj)
            %Creates a matrix of the knowledge region over state and action space
            
            [a,I1]=meshgrid(linspace(0,1,obj.agent.policy.numTests));
            for i=1:obj.agent.policy.numTests
                for j=1:obj.agent.policy.numTests
                    q(j,i)=obj.agent.kn.sim([a(i,j);I1(i,j)]);
                end
            end
            q=q./max(1,q);
            obj.KN=q;
            output=q;
        end
        

        
    end%methods
end%class